/** @file sp/spMemory.h
 */

#ifndef __SPMEMORY_H
#define __SPMEMORY_H

#include <sp/spDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spMemoryGroup Memory Management */

/** @addtogroup spMemoryGroup
 *  @{ */  /*----@addtogroup spMemoryGroup----*/

#ifdef MACOS
#undef SP_USE_MAC_MEMORY
#endif

#if defined(MACOS)
#pragma import on
#endif

extern void spSetMemoryErrorExit(int flag);
/** Allocates memory. */
extern char *xspMalloc(int nbytes);
/** Resize the allocated memory block. */
extern char *xspRemalloc(char *p, int nbytes);
/** Frees memory. Use xspFree() instead of this function. */
extern void _xspFree(char *p);

/** Allocates memory for a 2-d array of int type. */
extern int **xspIMatAlloc(int row, int col);
/** Frees memory of a 2-d array of int type. */
extern void xspIMatFree(int **mat, int row);
/** Allocates memory for a 2-d array of short type. */
extern short **xspSMatAlloc(int row, int col);
/** Frees memory of a 2-d array of short type. */
extern void xspSMatFree(short **mat, int row);
/** Allocates memory for a 2-d array of long type. */
extern long **xspLMatAlloc(int row, int col);
/** Frees memory of a 2-d array of long type. */
extern void xspLMatFree(long **mat, int row);
/** Allocates memory for a 2-d array of float type. */
extern float **xspFMatAlloc(int row, int col);
/** Frees memory of a 2-d array of float type. */
extern void xspFMatFree(float **mat, int row);
/** Allocates memory for a 2-d array of double type. */
extern double **xspDMatAlloc(int row, int col);
/** Frees memory of a 2-d array of double type. */
extern void xspDMatFree(double **mat, int row);

/** Allocates memory for a 2-d array of char type. */
extern char **xspCMatAlloc(int row, int col);
/** Frees memory of a 2-d array of char type. */
extern void xspCMatFree(char **mat, int row);
/** Allocates memory for a 2-d array of unsigned char type. */
extern unsigned char **xspUCMatAlloc(int row, int col);
/** Frees memory of a 2-d array of unsigned char type. */
extern void xspUCMatFree(unsigned char **mat, int row);

/** Makes a memory-allocated clone of the string specified by \p string . */
extern char *xspStrClone(const char *string);

#if defined(MACOS)
#pragma import off
#endif

#if defined(_WIN32) && defined(_DEBUG) && defined(_CRTDBG_MAP_ALLOC)
#define xspAlloc(n, type) (type *)malloc((int)(n)*sizeof(type))
#define xspRealloc(p, n, type) (type *)realloc((char *)(p), (int)(n)*sizeof(type))
#define xspFree(p) {free((char *)(p)); (p) = NULL;}
#define xspStrClone(p) _strdup(p)
#else
/** Macro to allocate memory. */
#define xspAlloc(n, type) (type *)xspMalloc((int)(n)*sizeof(type))
/** Macro to resize the allocated memory block. */
#define xspRealloc(p, n, type) (type *)xspRemalloc((char *)(p), (int)(n)*sizeof(type))
/** Macro to free the allocated memory block. */
#define xspFree(p) {_xspFree((char *)(p)); (p) = NULL;}
#endif
    
#define spStrReplace(s1, s2) {if ((s1) != NULL) xspFree(s1); (s1) = (((s2) != NULL) ? xspStrClone(s2) : NULL);}
/** Macro to copy the array contents of \p p2 to the address of \p p1 . */
#define spArrayCopy(p1, p2, n, type) memmove((char *)(p1), (char *)(p2), (unsigned)(n)*sizeof(type))

#if !defined(SP_DISABLE_ABBREVIATION)
#ifndef xalloc
#define xalloc(n, type) xspAlloc(n, type) /**< Macro to allocate memory (abbreviated version). */
#endif
#ifndef xrealloc
#define xrealloc(p, n, type) xspRealloc(p, n, type) /**< Macro to resize the allocated memory block (abbreviated version). */
#endif
#ifndef xfree
#define xfree(p) xspFree(p) /**< Macro to free the allocated memory block (abbreviated version). */
#endif

#ifndef strclone
#define strclone(string) xspStrClone(string) /**< Makes a memory-allocated clone of the string specified by \p string (abbreviated version). */
#endif
#ifndef strrepl
#define strrepl(s1, s2) spStrReplace(s1, s2)
#endif
#ifndef arrcpy
#define arrcpy(p1, p2, n, type) spArrayCopy(p1, p2, n, type) /**< Macro to copy the array contents of \p p2 to the address of \p p1 (abbreviated version). */
#endif
#endif /* !defined(SP_DISABLE_ABBREVIATION) */

/** @} */  /*----@addtogroup spMemoryGroup----*/

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPMEMORY_H */
